#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Liang Wang <liang.wang@cs.helsinki.fi>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.06.19
#


import os
import sys
import time

sys.path.append('../')
from panns import *

import numpy
from scipy import linalg


def test_big_data():
    rows, cols = 10000, 1000
    logger.info('start building %i x %i matrix ...' % (rows, cols))
    vecs = numpy.random.normal(0,1,(rows,cols))
    pidx = PannsIndex(cols, 'euclidean')
    logger.info('finish building  %i x %i matrix.' % (rows, cols))
    pidx.load_matrix(vecs)

    ntrees = 16
    logger.info('start building the index ...')
    pidx.parallelize(True)
    pidx.build(ntrees)
    logger.info('finish buliding the index.')

    logger.info('save index file ...')
    pidx.save('mytest.idx')
    pass


if __name__=='__main__':
    test_big_data()

    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Liang Wang <liang.wang@cs.helsinki.fi>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.01
#


import os
import sys
import mmap
import shutil
import logging
import cPickle as pickle
import multiprocessing

import h5py
import numpy
from scipy import linalg

from utils import *


logger = logging.getLogger('panns.pannsindex')


class PannsIndex():
    """
    This class builds up the index for the given data set. Two
    metrics are supported: Euclidean and Angular (cosine). The data
    set should be a matrix consisting of row vectors. For cosine, the
    data set is assumed normallized where data poit has length 1.
    """

    def __init__(self, dim=None, distance_metric='euclidean'):
        """
        Parameters:
        distance_metric: distance metric to use. euclidean or angular.
        """
        self.dim = dim    # dimension of data
        self.mtx = []     # list of row vectors
        self.btr = []     # list of binary-tree
        self.prj = []     # list of proj-planes
        self.K = 20       # Need to be tweaked
        self.parallel = False

        if distance_metric=='euclidean':
            self.metric = MetricEuclidean()
        elif distance_metric=='angular':
            self.metric = MetricCosine()
        pass


    def add_vector(self, v):
        """
        Add data vector to the current data set.
        """
        self.mtx.append(v)
        # Todo
        # if matrix size is bigger ... then
        pass


    def load_matrix(self, A):
        """
        Load data set from a row-based data matrix.
        """
        self.mtx = A
        pass


    def load_hdf5(self, fname, dataset='panns'):
        """
        Load data set from HDF5 file. Be careful that the performance
        of building up index will significantly degrade due to HDF5
        lookup overheads.

        Parameters:
        fname: file name of the HDF5 data set.
        """
        f = h5py.File(fname, mode='r', driver='core', libver='latest')
        self.mtx = f[dataset]
        pass


    def load_csv(self, fname, sep=','):
        """
        Load data set from a csv file.

        Parameters:
        fname: file name of the csv data set.
        sep:   the separator for the coordinates.
        """
        for line in open(fname, 'r'):
            v = [ numpy.float64(x) for x in line.split(sep) ]
            self.mtx.append(v)
        pass


    def mmap_core_data(self):
        """
        Convert mtx and prj to mmap file to save memory space, very
        useful when dealing with large dataset and parallel mode is
        activated. Skip if the data is already mmaped.
        """
        if type(self.mtx) != numpy.memmap:
            shape_mtx = (len(self.mtx), self.dim)
            mmap_mtx = make_mmap(self.mtx, shape_mtx)
            self.mtx = load_mmap(mmap_mtx, shape_mtx)
        if type(self.prj) != numpy.memmap:
            shape_prj = (len(self.prj), self.dim)
            mmap_prj = make_mmap(self.prj, shape_prj)
            self.prj = load_mmap(mmap_prj, shape_prj)
        pass


    def build(self, c=50):
        """
        Build the index for a given data set using random projections.
        The index is a forest of binary trees

        Parameters:
        c: the number of binary trees in the index.
        """
        num_prj = int(2 ** (numpy.log2(len(self.mtx) / self.K) + 1))
        self.prj = self.random_directions(num_prj)
        if self.parallel:
            self.mmap_core_data()
            num_cores = multiprocessing.cpu_count()
            pool = multiprocessing.Pool(num_cores)
            tbtr = [ pool.apply_async(build_parallel, [self.mtx.filename, self.prj.filename, self.mtx.shape, self.prj.shape, self.K, t]) for t in xrange(c) ]
            self.btr = [ r.get() for r in tbtr ]
            pool.terminate()
        else:
            self.build_sequential(c)
        pass


    def build_sequential(self, c):
        """
        Build the index sequentially, one projection at a time.

        Parameters:
        c: the number of binary trees to use in building the index.
        """
        for i in xrange(c):
            logger.info('pass %i ...' % i)
            tree = NaiveTree()
            self.btr.append(tree)
            children = range(len(self.mtx))
            self.make_tree(tree.root, children)
        pass


    def make_tree(self, parent, children, lvl=0):
        """
        The actual function which builds up a tree with recursion.

        Parameters:
        parent:   the index of the parent node.
        children: a list of children node indice.
        lvl:      mark the depth of the recursion.
        """
        if len(children) <= max(self.K, lvl):
            parent.nlst = children
            return
        l_child, r_child = None, None
        for attempt in xrange(16):
            parent.proj = numpy.random.randint(len(self.prj))
            u = self.prj[parent.proj]
            parent.ofst = self.metric.split(u, children, self.mtx)
            l_child, r_child = [], []
            for i in children:
                if self.metric.side(self.mtx[i], u, parent.ofst):
                    r_child.append(i)
                else:
                    l_child.append(i)
            if len(l_child) > 0 and len(r_child) > 0:
                break
        parent.lchd = Node()
        parent.rchd= Node()
        self.make_tree(parent.lchd, l_child, lvl+1)
        self.make_tree(parent.rchd, r_child, lvl+1)
        return


    def query(self, v, c):
        """
        Find the approximate nearest neighbors of a given vector.

        Parameters:
        v: the given vector.
        c: number of nearest neighbors.
        """
        r = set()
        for tree in self.btr:
            idxs = self.get_ann(tree.root, v, c)
            for idx in idxs:
                r.add( (idx, self.metric.distance(self.mtx[idx], v)) )
        r = list(r)
        r.sort(key = lambda x: x[1])
        return r[:c]


    def get_ann(self, p, v, c):
        """
        The function which finds the nearest neighbors recursively.

        Parameters:
        p: the parent node.
        v: the given vector.
        c: number of neighbors.
        """
        nns = None
        #if p.nlst is not None:
        if hasattr(p, 'nlst'):
            return p.nlst
        t = numpy.dot(self.prj[p.proj], v) - p.ofst
        if t > 0:
            nns = self.get_ann(p.rchd, v, c)
            if len(nns) < c:
                nns += self.get_ann(p.lchd, v, c)
        else:
            nns = self.get_ann(p.lchd, v, c)
            if len(nns) < c:
                nns += self.get_ann(p.rchd, v, c)
        return nns


    def principle_directions(self, c):
        """
        The function returns the principle components of a sample,
        then they are used as projection plane.

        Parameter:
        c: the number of principle components needed.
        """
        B = self.get_samples(max(c,1000))
        u,_,_ = linalg.svd(B.T)
        return u[:,:c].T


    def random_directions(self, c):
        """
        The function returns Gaussian random directions which are
        used as projection plane.

        Parameters:
        c: the number of principle components needed.
        """
        return [ gaussian_vector(self.dim, True) for _ in xrange(c) ]


    def get_samples(self, c):
        """
        Get a random sample from the data set.

        Parameters:
        c: number of points in the sample.
        """
        return self.mtx[:c]


    def linear_search(self, v, c):
        """
        Perform a linear search to find the exact nearest neighbors.

        Parameters:
        v: the given vector.
        c: number of nearest neighbors.
        """
        r = list()
        for i in xrange(len(self.mtx)):
            t = ( i, self.metric.distance(self.mtx[i],v) )
            r.append(t)
        r.sort(key = lambda x: x[1])
        return r[:c]


    def save(self, fname='panns.idx'):
        """
        The function saves the index in a file using cPickle.

        Parameters:
        fname: the index file name.
        """
        f = open(fname, 'wb')
        pickle.dump(self.basic_info(), f, -1)
        logger.info('dump random vectors to %s ...' % fname)
        pickle.dump(self.prj, f, -1)
        logger.info('dump binary trees to %s ...' % fname)
        for tree in self.btr:
            pickle.dump(tree, f, -1)
        logger.info('dump raw dataset to %s ...' % (fname+'.npy'))
        make_mmap(self.mtx, (len(self.mtx),self.dim), fname+'.npy')
        pass


    def load(self, fname):
        """
        Load the index into memory, mmap seems not working. :(

        Parameters:
        fname: The index file name.
        """
        f = open(fname, 'rb')
        mm = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
        d = pickle.load(mm)
        logger.info('load random vectors from %s...' % fname)
        self.prj = pickle.load(mm)
        logger.info('load binary trees from %s ...' % fname)
        self.btr = []
        while True:
            try:
                self.btr.append(pickle.load(mm))
            except:
                break
        logger.info('load raw dataset from %s ...' % (fname+'.npy'))
        self.mtx = numpy.memmap(fname+'.npy', dtype='float64', mode='r', shape=d['mtx_shape'])
        pass


    def parallelize(self, enable=False):
        """
        Enable the parallel building, one core for one tree.

        Parameters:
        enable: True is parallel, False is for sequential.
        """
        self.parallel = enable
        pass


    def basic_info(self):
        """
        Return a dict containing the basic info of the object.
        """
        d = dict()
        d['mtx_shape'] = (len(self.mtx), self.dim)
        return d


    def __del__(self):
        """
        Clean up the temp files and etc.
        """
        tmpdir = tempfile.gettempdir()
        if tmpdir is not None:
            try:
                shutil.rmtree(tmpdir)
            except:
                pass
        pass


    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Liang Wang <liang.wang@cs.helsinki.fi>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.01
#


import os
import sys
import time

sys.path.append('../')
from panns import *
from annoy import *

import numpy
from scipy import linalg


def random_norm_row_matrix(m, n, seed=None):
    if seed is not None:
        numpy.random.seed(seed)
    vecs = numpy.random.normal(0,1,(m,n))
    for i in xrange(m):
        vecs[i,:] /= linalg.norm(vecs[i,:])
    return vecs


def test_annoy(annoy, v, ans):
    t = time.time()
    n = annoy.get_nns_by_vector(v.tolist(), 10)
    r1 = precision(ans, n)
    r2 = recall(ans, n)
    print "Annoy precision: %.3f\t recall: %.3f\t %.4f ms" % (r1, r2, (time.time()-t)*1000)
    return r1, r2


def test_panns(panns, v, ans):
    t = time.time()
    n = panns.query(v, 10)
    r1 = precision(ans, n)
    r2 = recall(ans, n)
    print "Panns precision: %.3f\t recall: %.3f\t %.4f ms" % (r1, r2, (time.time()-t)*1000)
    return r1, r2


def compare_both(metric):
    m, n = 5000, 200
    print 'Build a %i x %i dataset ...' % (m, n)
    vecs = random_norm_row_matrix(m, n) # + 5.5

    num_prj = 128

    t = AnnoyIndex(n, metric)
    for i in xrange(m):
        t.add_item(i, vecs[i].tolist())
    t.build(num_prj)

    p = PannsIndex(n, metric)
    p.load_matrix(vecs)
    p.parallelize(True)
    p.build(num_prj)

    r1,r2,r3,r4 = 0,0,0,0
    for i in range(50):
        print '+'*30, 'test', i
        v = gaussian_vector(n, True)
        ans = p.linear_search(v, 10)

        t1, t2 = test_annoy(t, v, ans)
        t3, t4 = test_panns(p, v, ans)
        r1 += t1
        r2 += t2
        r3 += t3
        r4 += t4

    print '='*30, 'Summary', metric
    print "Annoy precision: %.3f\t recall: %.3f" % (r1/50, r2/50)
    print "Panns precision: %.3f\t recall: %.3f" % (r3/50, r4/50)

    t.save('annoy.'+metric+'.idx')
    p.save('panns.'+metric+'.idx')
    pass



if __name__=='__main__':
    compare_both('euclidean')
    compare_both('angular')
    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.20
#

import os
import sys
import time
import cPickle as pickle

sys.path.append('../')
from panns import *

import numpy
from scipy import linalg


def generate_large_matrix():
    rows, cols = 1000000, 500
    print 'Test serializing a %i x %i matrix ...' % (rows, cols)
    t = time.time()
    vecs = numpy.random.normal(0,1,(rows,cols))
    print 'Matrix constructed, spent %.2f s' % (time.time() - t)

    f1 = open('test_data1', 'wb')
    t = time.time()
    print 'saving as numpy npz format ...'
    numpy.savez_compressed(f1, vecs)
    print 'save done, spent %.2f s' % (time.time() - t)
    f1.close()

    f2 = open('test_data2', 'wb')
    t = time.time()
    print 'saving as self-defined format ...'
    for v in vecs:
        f2.write(pickle.dumps(v, -1))
    f2.close()
    print 'save done, spent %.2f s' % (time.time() - t)
    pass


def test_load_matrix(fn):
    print 'Loading matrix from self-defined format ...'
    f = open(fn, 'rb')
    mtx = []
    t = time.time()
    while True:
        try:
            v = pickle.load(f)
            mtx.append(v)
        except:
            break
    print 'The matrix loaded from self-defined format, spent %.2f s' % (time.time() - t)
    pass



if __name__=='__main__':
    generate_large_matrix()
    test_load_matrix('test_data2')
    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Liang Wang <liang.wang@cs.helsinki.fi>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.01
#


import os
import sys
import time

sys.path.append('../')
from panns import *

import numpy
from scipy import linalg


def test_add_vector():
    rows, cols = 50000, 50
    vecs = numpy.random.normal(0,1,(rows,cols))
    pidx = PannsIndex(cols, 'euclidean')
    print 'Test adding vector function by building %i x %i dataset ...' % (rows, cols)
    t = time.time()
    for v in vecs:
        pidx.add_vector(v)
    print 'Test is done in %.3f s' % (time.time() - t)
    pass


def test_index_file():
    rows, cols = 10000, 500
    print 'Build a %i x %i dataset ...' % (rows, cols)
    vecs = numpy.random.normal(0,1,(rows,cols))

    pidx = PannsIndex(cols, 'euclidean')
    pidx.load_matrix(vecs)
    pidx.parallelize(False)
    pidx.build(16)

    t = time.time()
    v = gaussian_vector(cols, True)
    r1 = pidx.linear_search(v, 10)
    print 'Linear search is done in %.3f s' % (time.time()-t)
    r2 = pidx.query(v, 10)
    m1 = precision(r1, r2)
    m2 = recall(r1, r2)
    print "Precision: %.3f, Recall: %.3f" % (m1, m2)

    t = time.time()
    print 'Test saving the index files ...'
    pidx.save('mytest.idx')
    print 'Saving is done in %.3f s' % (time.time()-t)

    print 'Test loading the index and querying ...'
    t = time.time()
    pidx.load('mytest.idx')
    print 'Loading is done in %.3f s' % (time.time()-t)

    t = time.time()
    r = pidx.query(v, 10)
    print 'Test querying done in %.3f s' % (time.time()-t)
    pass


def test_parallel_build():
    print 'Test building index in parallel mode ...'
    rows, cols = 100000, 50
    print 'Build a %i x %i dataset ...' % (rows, cols)
    vecs = numpy.random.normal(0,1,(rows,cols))

    pidx = PannsIndex(cols)
    pidx.load_matrix(vecs)
    pidx.parallelize(True)
    pidx.build(100)
    print 'Parallel bulding is done.'
    v = gaussian_vector(cols, True)
    r1 = pidx.linear_search(v, 10)
    r2 = pidx.query(v, 10)
    m1 = precision(r1, r2)
    m2 = recall(r1, r2)
    print "Precision: %.3f, Recall: %.3f" % (m1, m2)
    pass


def test_load_csv(fname):
    pidx = PannsIndex(50)
    pidx.load_csv(fname, ',')
    print pidx.mtx
    pass


if __name__=='__main__':
    ###test_add_vector()
    test_index_file()
    ###test_parallel_build()
    ###test_load_csv('zzz')
    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Liang Wang <liang.wang@cs.helsinki.fi>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.01
#


import logging
import numpy
import tempfile
from scipy import linalg
from scipy.spatial import distance


logger = logging.getLogger('panns.utils')


class Node():
    __slots__ = ['proj', 'ofst', 'lchd', 'rchd', 'nlst']
    pass


class NaiveTree(object):

    def __init__(self):
        self.root = Node()
        pass

    pass


class Metric():
    """
    The basic metric class used in Panns index building. Super class
    of MetricEuclidean and MetricCosine.
    """

    @staticmethod
    def split(u, idxs, mtx):
        """
        Project the data points on a random vector and return the
        average value.

        Parameters:
        u:    random vector.
        idxs: data points to project.
        mtx:  data set.
        """
        v = numpy.zeros(len(u), dtype='float64')
        for i in idxs:
            v += mtx[i]
        a = numpy.dot(u, v) / len(idxs)
        return a


    @staticmethod
    def side(u, v, offset):
        """
        Project v on u then check which side it falls in given the
        offset.

        Parameters:
        u: random vector.
        v: data point to project.
        """
        r = None
        x = numpy.dot(u, v) - offset
        if abs(x) < 1e-12:
            r = ( numpy.random.uniform(0,1,1)[0] > 0.5 )
        else:
            r = ( x > 0 )
        return r

    pass


class MetricEuclidean(Metric):
    """
    Metric class for Euclidean index.
    """

    @staticmethod
    def distance(u, v):
        return distance.euclidean(u, v)

    pass


class MetricCosine(Metric):
    """
    Metric class for cosine index.
    """

    @staticmethod
    def distance(u, v):
        return 1.0 - numpy.dot(u, v)

    pass


def gaussian_vector(size, normalize=False):
    """
    Returns a (normalized) Gaussian random vector.

    Parameters:
    normalize: the vector length is normalized to 1 if True.
    """
    v = numpy.random.normal(0,1,size)
    if normalize:
        v = v / linalg.norm(v)
    return v


def precision(relevant, retrieved):
    """
    Return the precision of the search result.

    Parameters:
    relevant:  the relevant data points.
    retrieved: the retireved data points
    """
    r = 1.0 * len(set(relevant) & set(retrieved)) / len(retrieved)
    return r


def recall(relevant, retrieved):
    """
    Return the recall of the search result.

    Parameters:
    relevant:  the relevant data points.
    retrieved: the retireved data points
    """
    r = 1.0 * len(set(relevant) & set(retrieved)) / len(relevant)
    return r


def build_parallel(mtx, prj, shape_mtx, shape_prj, K, t):
    """
    The function for parallel building index. Implemented here because
    the default python serialization cannot pickle instance function.

    Parameters:
    mtx: a row-based data set.
    K:   max number of data points on a leaf.
    t:   index of binary trees.
    """
    logger.info('pass %i ...' % t)
    mtx = numpy.memmap(mtx, dtype='float64', mode='r', shape=shape_mtx)
    prj = numpy.memmap(prj, dtype='float64', mode='r', shape=shape_prj)
    numpy.random.seed(t**2)
    tree = NaiveTree()
    children = range(len(mtx))
    make_tree_parallel(tree.root, children, mtx, prj, K)
    return tree


def make_tree_parallel(parent, children, mtx, prj, K, lvl=0):
    """
    Builds up a binary tree recursively, for parallel building.

    Parameters:
    parent:   parent node index.
    children: a list of children node indices.
    mtx:      a row-based data set.
    K:        max number of data points on a leaf.
    """
    if len(children) <= max(K, lvl):
        parent.nlst = children
        return
    l_child, r_child = None, None
    for attempt in xrange(16):
        parent.proj = numpy.random.randint(len(prj))
        u = prj[parent.proj]
        parent.ofst = Metric.split(u, children, mtx)
        l_child, r_child = [], []
        for i in children:
            if Metric.side(mtx[i], u, parent.ofst):
                r_child.append(i)
            else:
                l_child.append(i)
        if len(l_child) > 0 and len(r_child) > 0:
                break
    parent.lchd = Node()
    parent.rchd = Node()
    make_tree_parallel(parent.lchd, l_child, mtx, prj, K)
    make_tree_parallel(parent.rchd, r_child, mtx, prj, K)
    return


def make_mmap(mtx, shape, fname=None):
    m, n  = shape
    if fname is None:
        fname = tempfile.mkstemp()[1]
    logger.info('mmaping the data to %s ...' % fname)
    fpw = numpy.memmap(fname, dtype='float64', mode='w+', shape=(m,n))
    for i in xrange(m):
        fpw[i] = mtx[i]
    del fpw
    return fname


def load_mmap(fname, shape):
    mtx = numpy.memmap(fname, dtype='float64', mode='r', shape=shape)
    return mtx



from distutils.core import setup

VER = '0.1.4'

setup(
    name='panns',
    version=VER,
    packages=['panns',],
    description = 'Python Approximate Nearest Neighbor / Neighbour Search',
    #long_description=open('README.md').read(),
    author = 'Liang Wang',
    author_email = 'ryanrhymes@gmail.com',
    url = 'https://github.com/ryanrhymes/panns',
    download_url = 'https://github.com/ryanrhymes/panns/archive/%s.zip' % VER,
    keywords = ['k-nn', 'knn', 'nearest neighbor', 'nearest neighbor search' 'approximate nearest neighbor', 'approximate neighbor search',
                'nearest neighbour', 'nearest neighbour search' 'approximate nearest neighbour', 'approximate neighbour search',
                'high demensional search', 'high-dimension data', 'LSH', 'locality sensitive hashing', 'random projection'],
    classifiers = [],
    license='GNU LGPL v2.1',

)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.20
#

import os
import sys
import time

sys.path.append('../')
from panns import *

import h5py
import numpy


def test_iterate_performance():
    t = time.time()
    m, n = 10000, 500
    mtx = numpy.random.normal(0, 1, (m, n))
    print 'Matrix is generated, spent %.3f s' % (time.time()-t)

    f = h5py.File('test.hdf5', mode='w', driver='core', libver='latest')
    dset = f.create_dataset('panns', (m, n), dtype='float')
    t = time.time()
    for i in xrange(m):
        dset[i,:] = mtx[i,:]
    f.close()
    print 'Iteration-based update is done in %.3f s' % (time.time()-t)
    pass


def test_block_performance():
    t = time.time()
    m, n = 10000, 500
    mtx = numpy.random.normal(0, 1, (m, n))
    print 'Matrix is generated, spent %.3f s' % (time.time()-t)

    f = h5py.File('test.hdf5', mode='w', driver='core', libver='latest')
    dset = f.create_dataset('panns', (m, n), dtype='float64')
    t = time.time()
    dset[:] = mtx[:]
    f.close()
    print 'Block-based update is done in %.3f s' % (time.time()-t)
    pass


def test_lookup_performance(fn):
    f = h5py.File(fn, mode='r', driver='core', libver='latest')
    dset = f['panns']
    m, n = dset.shape
    z = numpy.zeros((1,n))
    t0, c = 0.0, 100000
    for i in xrange(c):
        t1 = time.time()
        idx = numpy.random.randint(m)
        z = dset[idx,:]
        t0 += time.time() - t1
    print 'Average lookup time is %.2f ms' % (t0 * 1000 / c)
    pass


def test_building_index(fn):
    p = PannsIndex(500, 'euclidean')

    mtx = numpy.random.normal(0,1,(10000,500))
    p.load_matrix(mtx)
    t = time.time()
    p.build(5)
    print 'Mem-based building took %.3f s' % (time.time()-t)

    p.load_hdf5(fn)
    t = time.time()
    p.build(5)
    print 'HDF5-based building took %.3f s' % (time.time()-t)
    pass


if __name__=='__main__':
    test_iterate_performance()
    #test_block_performance()
    #test_lookup_performance('test.hdf5')
    test_building_index('test.hdf5')
    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 Liang Wang <liang.wang@cs.helsinki.fi>
# Licensed under the GNU LGPL v2.1 - http://www.gnu.org/licenses/lgpl.html
#
# Liang Wang @ CS Dept, Helsinki Univ, Finland
# 2014.05.31
#


from .index import PannsIndex
from .utils import *

import logging
logger = logging.getLogger('panns')
logging.basicConfig(format='%(asctime)s : %(levelname)s : #%(process)d => %(message)s', level=logging.INFO)
